/* Copyright 2004, 2005 Nicholas Bishop
 * 
 * This file is part of SharpConstruct.
 *
 * SharpConstruct is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * SharpConstruct is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SharpConstruct; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#ifndef BRUSH_H
#define BRUSH_H

#include "Color.h"

namespace SharpConstruct
{
	class Brush
	{
	public:
		enum ModelingMode{ DISPLACE, GRAB, SMOOTH };	
	
		Brush();
		~Brush();
		
		void SetModelingMode( ModelingMode );
		ModelingMode GetModelingMode() const;
		
		void SetPositionEdit( bool );
		bool GetPositionEdit() const;
		
		void SetColorEdit( bool );
		bool GetColorEdit() const;
		
		void SetRadius( float );
		void AdjustRadius( float );
		float GetRadius() const;
		
		void SetStrength( float );
		float GetStrength() const;
		
		void SetIntensity( float );
		float GetIntensity() const;
		
		const Color& GetColor() const
		{
			return _color;
		}
		void SetColor( const Color& c )
		{
			_color = c;
		}
		
		void SetRed( float );
		float GetRed() const;
		
		void SetGreen( float );
		float GetGreen() const;
		
		void SetBlue( float );
		float GetBlue() const;
		
		void SetSymmetryX( bool );
		bool GetSymmetryX() const;
		
		void SetSymmetryY( bool );
		bool GetSymmetryY() const;
		
		void SetSymmetryZ( bool );
		bool GetSymmetryZ() const;
	private:
		ModelingMode _modeling_mode;
		bool _position_edit, _color_edit;
		float _radius, _strength, _intensity;
		Color _color;
		bool _symmetry_x, _symmetry_y, _symmetry_z;
	};
}

#endif // BRUSH_H
