/* Copyright 2005 Nicholas Bishop
 * 
 * This file is part of SharpConstruct.
 *
 * SharpConstruct is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * SharpConstruct is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SharpConstruct; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#ifndef COLORPICKER_H
#define COLORPICKER_H

#include "Color.h"
#include "Group.h"

namespace SharpConstruct
{
	namespace Interface
	{
		// Does nothing but display its color
		class ColorSwatch : public Widget
		{
		public:
			ColorSwatch();
		
			virtual ~ColorSwatch()
			{}
				
			void SetColor( const Color& c );
		protected:
			virtual void _draw();			
		private:
			Color _color;
		};
		
		class ValueIntensityPicker : public Widget
		{
		public:
			ValueIntensityPicker();
				
			virtual ~ValueIntensityPicker()
			{}
			
			Color FinalColor() const;
			void SetFinalColor( Color c );
			void SetBaseColor( const Color& c );
				
			sigc::signal< void, Color > ChangedSignal;
		protected:
			virtual void _draw();
		private:
			void _set();	

			SharpConstruct::Color _base_color, _final_color;
			Math::Point2D< int > _selection;
		};
		
		class HuePicker : public Widget
		{
		public:
			HuePicker();	
		
			virtual	~HuePicker()
			{}
				
			void SetColor( Color c );
			
			sigc::signal< void, Color > ChangedSignal;
		protected:
			virtual void _draw();
		private:
			void _set();
		
			SharpConstruct::Color _color;
			Math::Point2D< int > _selection;
		};
		
		class ColorPicker : public Group
		{
		public:
			ColorPicker( int h );
				
			virtual ~ColorPicker()
			{}
			
			Color FinalColor() const;
			void SetColor( const Color& c );
				
			sigc::signal< void, Color > ChangedSignal;
		protected:
			//virtual void _draw();
		private:
			HuePicker _hue_picker;
			ValueIntensityPicker _vi_picker;
			ColorSwatch _swatch;
		};
	}
}

#endif // COLORPICKER_H
