/* Copyright 2005 Nicholas Bishop
 * 
 * This file is part of SharpConstruct.
 *
 * SharpConstruct is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * SharpConstruct is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SharpConstruct; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#ifndef DIALOG_H
#define DIALOG_H

#include "Button.h"
#include "Group.h"
#include "Window.h"

namespace SharpConstruct
{
	namespace Interface
	{
		class Dialog : public Window< VerticalGroup >
		{
		public:
			Dialog( std::string label );
				
			virtual ~Dialog()
			{}
				
			virtual void Reorganize();

			virtual void DoEvents();
			virtual bool Close();

			virtual void Cancel();
				
			virtual void Show();
			virtual void MouseDown();
			
			sigc::signal< void > CancelSignal;
			sigc::signal< void, std::string > DoneSignal;
			sigc::signal< void, Dialog* > RaiseSignal;
		protected:
			virtual void _draw();
		private:
			void _mouse_down();
			void _mouse_move();
		
			Math::Point2D< int > _relative_click_location;
			Button _title; // Ick?
			Group _close_area;
			Button _close;
		};
		
		class DialogList : public Group
		{
		public:
			DialogList();

			virtual void PushBack( Dialog& d );
			virtual void PushBack( Dialog* d );
		private:
			void _raise( Dialog* d );
		};
	}
}

#endif // DIALOG_H
