/* Copyright 2005 Nicholas Bishop
 * 
 * This file is part of SharpConstruct.
 *
 * SharpConstruct is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * SharpConstruct is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SharpConstruct; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#ifndef FILEDIALOG_H
#define FILEDIALOG_H

#include "Button.h"
#include "Dialog.h"
#include "Divider.h"
#include "ListBox.h"
#include "TextEntry.h"
#include "Toggle.h"

namespace SharpConstruct
{
	namespace Interface
	{
		class FileDialog : public Dialog
		{
		public:
			FileDialog( std::string label );
				
			virtual ~FileDialog()
			{}
			
			void ChangeDir( std::string );
			void RefreshEntries();

			virtual void Reorganize();
			
			static std::string Trailing( std::string path );
			static std::string PathPart( std::string path );
			static std::string FilePart( std::string path );
			static std::string HomePath();
			static bool FileExists( std::string path );
				
			std::string GetPath();
		protected:
			virtual void _done();
			HorizontalGroup _control_buttons;
		private:
			void _change_dir();
			void _go_root();
			void _go_home();
			void _go_up();
			void _make_dir();
			void _set_file( std::string file );
			
			const int _DEFAULT_HEIGHT;
			
			HorizontalGroup _dir_controls;
			Button _root;
			Button _home;
			Button _new_dir;
			Toggle _hidden;
		
			HorizontalGroup _main;
			VerticalGroup _dirs_list_group;
			ListBox _dirs;
			Button _dir_up;
			VerticalDivider _dir_file_split;
			ListBox _files;
		
			TextEntry _path_entry;
			//HorizontalGroup _control_buttons;
			Button _cancel;

			std::string _dir;
		};
	}
}
#endif // FILEDIALOG_H
