/* Copyright 2005 Nicholas Bishop
 * 
 * This file is part of SharpConstruct.
 *
 * SharpConstruct is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * SharpConstruct is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SharpConstruct; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#ifndef GROUP_H
#define GROUP_H

#include "Widget.h"
#include "WidgetList.h"
#include <vector>

namespace SharpConstruct
{
	namespace Interface
	{
		class Group : public Widget, public List< Widget* >
		{
		public:
			Group( std::string lbl )
			: Widget( lbl ), _corporeal( true ), _internal_border( 0 ),
			  _size_managed( false )
			{
				SetBorder( 0 );
			}
			Group( int brd = 0 )
			: Widget( brd ), _corporeal( true ), _internal_border( 0 ),
			  _size_managed( false )
			{}
			Group( std::string label, int brd )
			: Widget( label, brd ), _corporeal( true ), _internal_border( 0 ),
			  _size_managed( false )
			{}
			Group( int w, int h, std::string label )
			: Widget( w, h, label ), _corporeal( true ), _internal_border( 0 ),
			  _size_managed( false )
			{}
			Group( int x, int y, int w, int h, std::string label )
			: Widget( x, y, w, h, label ), _corporeal( true ),
			  _internal_border( 0 ), _size_managed( false )
			{}
				
			virtual ~Group()
			{}

			void DrawChildren();
				
			virtual void Reorganize();
				
			virtual void MouseDown();
			virtual void MouseStillDown();
			virtual void MouseUp();
			virtual void DoubleClick();
			virtual void MouseMove();
			virtual void ScrollDown();
			virtual void ScrollUp();
			virtual void KeyDown();
			virtual bool DoShortcut();

			/*virtual void MoveX( int x );
			virtual void MoveY( int y );
			virtual void MoveRight( int r );
			virtual void MoveBottom( int b );*/
			virtual void SetX( int x );
			virtual void SetY( int y );
			virtual void SetRight( int r );
			virtual void SetBottom( int b );
			virtual void SetW( int w );
			virtual void SetH( int h );
			
			bool Corporeal() const;
			void SetCorporeal( bool c );
			int InternalBorder() const;
			void SetInternalBorder( int brd );
			bool SizeManaged() const;
			void SetSizeManaged( bool m );
			
			virtual void PushFront( Widget& w );
			virtual void PushFront( Widget* w );
			virtual void PushBack( Widget& w );
			virtual void PushBack( Widget* w );
			virtual void Erase( Iterator i );
		protected:
			virtual void _draw();
		private:
			bool _corporeal;
			int _internal_border;
			bool _size_managed;
		};
		
		class HorizontalGroup : public Group
		{
		public:
			HorizontalGroup( bool m = false, int brd = 0, int ibrd = 0 )
			: Group( brd )
			{
				SetInternalBorder( ibrd );
				SetSizeManaged( m );
			}
			
			virtual void Reorganize();
		};
		
		class VerticalGroup : public Group
		{
		public:
			VerticalGroup( bool m = true, int brd = 0, int ibrd = 0 )
			: Group( brd )
			{
				SetInternalBorder( ibrd );
				SetSizeManaged( m );
			}
			
			virtual void Reorganize();
		};
	}
}

#endif // GROUP_H
