/* Copyright 2004, 2005 Nicholas Bishop
  
  This file is part of SharpConstruct.
 
  SharpConstruct is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
 
  SharpConstruct is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with SharpConstruct; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#ifndef MAINWINDOW_H
#define MAINWINDOW_H

//#include "AboutDialog.h"
#include "Button.h"
#include "ColorPicker.h"
#include "DeformDialog.h"
#include "Infobar.h"
#include "MeshHistory.h"
#include "NewDialog.h"
#include "OpenDialog.h"
#include "SaveDialog.h"
#include "SettingsDialog.h"
#include "Slider.h"
#include "Toggle.h"
#include "TopWindow.h"
#include "View.h"

namespace SharpConstruct
{
	namespace Interface
	{
		class MainWindow : public TopWindow
		{
		public:
			MainWindow();
			~MainWindow();
			
			void Refresh();
			void RefreshAll();
			virtual bool Close();
		
			virtual void Reorganize();
			virtual void Resize();
		
			bool IsMeshDataSafe();
		
			MeshHistory& GetMeshHistory();
			ViewGL& View();
			void SetStatusText( std::string t );
			void DisplayModelInfo();
			void SetModelPath( std::string p );
		protected:
			virtual void _draw();
		private:
			void _initialize_view();
			void _initialize_widgets();
		
			void _display_about();
		
			// Mesh data
			MeshHistory _mesh_history;
			SharpConstruct::Brush _brush;
		
			// OpenGL view
			SharpConstruct::Interface::ViewGL _v;
			// Dialogs
			//AboutDialog _about_dialog;
			DialogList _dialogs;
			DeformDialog _deform_dialog;
			NewDialog _new_dialog;
			OpenDialog _open_dialog;
			SaveDialog _save_dialog;
			SettingsDialog _settings_dialog;

			HorizontalGroup _top_bar;
		
			HorizontalGroup _model;
			VerticalGroup _model_left;
			VerticalGroup _model_right;
		
			HorizontalGroup _brush_settings;
			VerticalGroup _brush_settings_mode;
			VerticalGroup _brush_settings_sliders;
			HorizontalGroup _brush_settings_push;
			VerticalGroup _brush_settings_symmetry;
			
			//Group _deform;
			Group _texture;
			//Group _render;
			VerticalGroup _view;
			Group _preferences;
			// Model controls
			Button _load;
			Button _new;
			Button _save;
			Button _save_as;
			Button _undo;
			Button _redo;
			// Modeling controls
			Toggle _displace;
			Toggle _grab;
			Toggle _smooth;
			Button _deform;
			Slider _radius;
			Slider _strength;
			Slider _intensity;
			Toggle _add;
			Toggle _sub;
			Toggle _rgb;
			Toggle _x_mirror;
			Toggle _y_mirror;
			Toggle _z_mirror;
			ColorPicker _color;
			// Other controls
			Toggle _smoothed;
			Toggle _double_sided;
			Button _settings;
		
			HorizontalGroup _bottom_bar;
			Button _about;
			Button _model_info;
			Infobar _status_bar;
			
			bool _open_model( std::string path );
			void _save_model();
			void _save_model_as( std::string );
			bool _dialog_save_model();
			bool _dialog_dontsave_model();
		
			// Callbacks
			
			void _cb_view();
			void _cb_texture_group();

			void _cb_undo();
			void _cb_redo();
			
			void _cb_draw();
			void _cb_grab();
			void _cb_smooth();
			void _cb_update_brush_settings();
			void _cb_add();
			void _cb_sub();
			void _cb_rgb();
			void _cb_symmetry();
			
			void _cb_about();
			
			// Settings
			void _load_preferences();
			void _save_preferences();
			std::string _model_path;
			std::string _last_used_texture_path;
		};
	}
}

#endif // MAINWINDOW_H
