/* Copyright 2004, 2005 Nicholas Bishop
 * 
 * This file is part of SharpConstruct.
 *
 * SharpConstruct is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * SharpConstruct is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SharpConstruct; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#ifndef MATH_H
#define MATH_H

#include "Optimized.h"

namespace SharpConstruct
{
	namespace Math
	{
		template< class Coord > class Point2D
		{
		public:
			Point2D() : _x( 0 ), _y( 0 )
			{}
			Point2D( Coord x, Coord y ) : _x( x ), _y( y )
			{}
				
			Coord X() const
			{
				return _x;
			}
			Coord Y() const
			{
				return _y;
			}
			void SetX( Coord x )
			{
				_x = x;
			}
			void SetY( Coord y )
			{
				_y = y;
			}
			void Set( Coord x, Coord y )
			{
				SetX( x );
				SetY( y );
			}				
		private:
			Coord _x, _y;
		};
		
		template< class Coord > class Rect2D
		{
		public:
			Rect2D( Coord x, Coord y, Coord w, Coord h )
			: _loc( x, y ), _w( w ), _h( h )
			{}
			
			Point2D< Coord > Location() const
			{
				return _loc;
			}
			Coord X() const
			{
				return Location().X();
			}
			Coord Y() const
			{
				return Location().Y();
			}
			Coord Right() const
			{
				return X() + W() - 1;
			}
			Coord Bottom() const
			{
				return Y() + H() - 1;
			}
			Coord W() const
			{
				return _w;
			}
			Coord H() const
			{
				return _h;
			}
			
			virtual void AnchoredSetX( Coord x )
			{
				Coord tR = Right();
				SetX( x );
				SetRight( tR );
				
			}
			virtual void AnchoredSetY( Coord y )
			{
				Coord tB = Bottom();
				SetY( y );
				SetBottom( tB );
			}
			virtual void AnchoredSetRight( Coord r )
			{
				_w = r - X() + 1;
			}
			virtual void AnchoredSetBottom( Coord b )
			{
				_h = b - Y() + 1;
			}
			
			virtual void SetX( Coord x )
			{
				_loc.SetX( x );
			}
			virtual void SetY( Coord y )
			{
				_loc.SetY( y );
			}
			virtual void SetRight( Coord r )
			{
				SetX( X() + r - Right() );
			}
			virtual void SetBottom( Coord b )
			{
				SetY( Y() + b - Bottom() );
			}
			virtual void SetW( Coord w )
			{
				_w = w;
			}
			virtual void SetH( Coord h )
			{
				_h = h;
			}
			virtual void SetSize( Coord w, Coord h )
			{
				SetW( w );
				SetH( h );
			}
			virtual void Set( Coord x, Coord y, Coord w, Coord h )
			{
				SetX( x );
				SetY( y );
				SetW( w );
				SetH( h );
			}
	
			Point2D< Coord > Center() const
			{
				return Point2D< Coord >( X() + W() / 2, Y() + H() / 2 );
			}
			
			virtual bool Contains( Point2D< Coord > t ) const
			{
				return t.X() >= X() && t.Y() >= Y() &&
				       t.X() <= Right() && t.Y() <= Bottom();
			}
		private:
			Point2D< Coord > _loc;
			Coord _w, _h;
		};
	}
}
#endif // MATH_H
