/* Copyright 2005 Nicholas Bishop
 * 
 * This file is part of SharpConstruct.
 *
 * SharpConstruct is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * SharpConstruct is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SharpConstruct; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#ifndef SCROLLBAR_H
#define SCROLLBAR_H

#include "Button.h"
#include "Group.h"

namespace SharpConstruct
{
	namespace Interface
	{
		class Scrollbar : public Group
		{
		public:		
			Scrollbar( int x, int y, int w, int h );
				
			virtual ~Scrollbar()
			{}
			
			int Value() const
			{
				return _value;
			}
			void SetValue( int v )
			{
				if( v > _max - 1 )
					v = _max - 1;
				else if( v < 0 )
					v = 0;
				
				_value = v;
				ChangedSignal.emit();
			}
			void Decrement()
			{
				SetValue( Value() - 1 );
			}				
			void Increment()
			{
				SetValue( Value() + 1 );
			}
			int Max()
			{
				return _max;
			}
			void SetMax( int m )
			{
				if( m < 1 )
					m = 1;
				_max = m;
				SetValue( Value() );
			}
			
			sigc::signal< void > ChangedSignal;
		protected:
			virtual void _mouse_down();
		
			Button _up, _down;
		private:		
			int _max, _value;
		};
		
		class VerticalScrollbar : public Scrollbar
		{
		public:
			VerticalScrollbar( int x, int y, int w, int h );
		protected:
			virtual void _draw();
		protected:
			void _mouse_down();
		};
	}
}
#endif // SCROLLBAR_H
