/* Copyright 2005 Nicholas Bishop
 * 
 * This file is part of SharpConstruct.
 *
 * SharpConstruct is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * SharpConstruct is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SharpConstruct; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#ifndef SLIDER_H
#define SLIDER_H

#include "Widget.h"

namespace SharpConstruct
{
	namespace Interface
	{
		class Slider : public Widget
		{
		public:
			Slider( int x, int y, int w, int h, std::string label )
			: Widget( x, y, w, h, label ), _min( 0 ), _max( 10 ), _step( 1 ),
			  _value( 0 ), _anchor( 0 ), _steady( true )
			{
				MouseMoveSignal.connect( sigc::mem_fun( *this, &Slider::_mouse_move ) );
				MouseDownSignal.connect( sigc::mem_fun( *this, &Slider::_mouse_down ) );
				MouseUpSignal.connect( sigc::mem_fun( *this, &Slider::_mouse_up ) );
				SetCursor( HORIZ_ARROWS );
			}
			Slider( int w, int h, std::string label )
			: Widget( w, h, label ), _min( 0 ), _max( 10 ), _step( 1 ),
			  _value( 0 ), _anchor( 0 ), _steady( true )
			{
				MouseMoveSignal.connect( sigc::mem_fun( *this, &Slider::_mouse_move ) );
				MouseDownSignal.connect( sigc::mem_fun( *this, &Slider::_mouse_down ) );
				MouseUpSignal.connect( sigc::mem_fun( *this, &Slider::_mouse_up ) );
				SetCursor( HORIZ_ARROWS );
			}
			Slider( int w, std::string label )
			: Widget( w, CONTROL_HEIGHT, label ),
			  _min( 0 ), _max( 10 ), _step( 1 ), _value( 0 ), _anchor( 0 ),
			  _steady( true )
			{
				MouseMoveSignal.connect( sigc::mem_fun( *this, &Slider::_mouse_move ) );
				MouseDownSignal.connect( sigc::mem_fun( *this, &Slider::_mouse_down ) );
				MouseUpSignal.connect( sigc::mem_fun( *this, &Slider::_mouse_up ) );
				SetCursor( HORIZ_ARROWS );
			}
			Slider( std::string label )
			: Widget( SLIDER_WIDTH, CONTROL_HEIGHT, label ),
			  _min( 0 ), _max( 10 ), _step( 1 ), _value( 0 ), _anchor( 0 ),
			  _steady( true )
			{
				MouseMoveSignal.connect( sigc::mem_fun( *this, &Slider::_mouse_move ) );
				MouseDownSignal.connect( sigc::mem_fun( *this, &Slider::_mouse_down ) );
				MouseUpSignal.connect( sigc::mem_fun( *this, &Slider::_mouse_up ) );
				SetCursor( HORIZ_ARROWS );
			}
				
			virtual ~Slider()
			{}
			
			virtual void KeyDown();
			virtual void MakeInactive();
			
			void SetProperties( float min, float max, float step )
			{
				if( min < max )
				{
					_min = min;
					_max = max;
				}
				else
				{
					_min = max;
					_max = min;
				}
				
				_anchor = _min;
				
				_step = fabs( step );
				Value( Value() );
			}
			void SetProperties( float min, float max, float step, float anchor,
			                    bool steady );
			float Anchor() const;
			void SetAnchor( float a );
			bool Steady() const;
			void SetSteady( bool s );
			
			float Range() const
			{
				return _max - _min;
			}
			float LocationOf( float v ) const;
			float Value() const
			{
				return _value;
			}
			void Value( float v );

			sigc::signal< void, float > ChangedSignal;
			sigc::signal< void > SetSignal;
		protected:
			virtual void _draw();
		private:
			void _mouse_down();
			void _mouse_up();
			void _mouse_move();
		
			float _min, _max, _step, _value, _anchor;
			bool _steady;
			float _orig;
		
			int _sel_start;
			bool _sel_all;
		};
	}
}
#endif // SLIDER_H
