/* Copyright 2005 Nicholas Bishop
 * 
 * This file is part of SharpConstruct.
 *
 * SharpConstruct is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * SharpConstruct is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SharpConstruct; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#include "Math.h"
#include <string>
#include <vector>

namespace SharpConstruct
{
	namespace GLFormat
	{
		class Align
		{
		public:
			class Left
			{};
			class Center
			{};
			class Right
			{};
			enum Alignment
			{
				AlignLeft,
				AlignCenter,
				AlignRight
			};
		};
		typedef Math::Point2D< int > Location;
	}
	
	class GLOut;
	
	GLOut& operator<<( GLOut&, std::basic_string< char,
	                           std::char_traits< char >,
		                       std::allocator< char > > );
	
	class Font
	{
	public:
		Font( std::string );
		unsigned W() const
		{
			return _w;
		}
		unsigned H() const
		{
			return _h;
		}
	private:
		std::string _name;
		unsigned _w, _h;
		unsigned _id;
	};
	
	class GLOut
	{
		friend GLOut& SharpConstruct::operator<<( GLOut&, std::basic_string< char, std::char_traits< char >,
		                   std::allocator< char > > );
		
	public:
		GLOut();
		
		GLOut& operator<<( GLFormat::Location loc )
		{
			_location = loc;
			return *this;
		}
		GLOut& operator<<( GLFormat::Align::Left a )
		{
			_alignment = GLFormat::Align::AlignLeft;
			return *this;
		}
		GLOut& operator<<( GLFormat::Align::Center a )
		{
			_alignment = GLFormat::Align::AlignCenter;
			return *this;
		}
		GLOut& operator<<( GLFormat::Align::Right a )
		{
			_alignment = GLFormat::Align::AlignRight;
			return *this;
		}
		GLOut& operator<<( float );
		
		const Font& CurrentFont() const;
		
		static GLOut glout;
	private:
		void _load_fonts();
		void _draw_text( std::string );
		void _draw_letter( char );
	
		std::vector< Font > _font;
		GLFormat::Location _location;
		GLFormat::Align::Alignment _alignment;
		unsigned _current_font;
	
		//
		unsigned id;
	};
}
