/* Copyright 2005 Nicholas Bishop
 * 
 * This file is part of SharpConstruct.
 *
 * SharpConstruct is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * SharpConstruct is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SharpConstruct; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#ifndef TOGGLE_H
#define TOGGLE_H

#include "Widget.h"

namespace SharpConstruct
{
	namespace Interface
	{
		class Toggle : public Widget
		{
		public:
			Toggle( int x, int y, int w, int h, std::string lbl )
			: Widget( x, y, w, h, lbl ), _pressed( false )
			{
				MouseUpSignal.connect( sigc::mem_fun( *this, &Toggle::PressToggle ) );
			}
			Toggle( int w, int h, std::string lbl )
			: Widget( w, h, lbl ), _pressed( false )
			{
				MouseUpSignal.connect( sigc::mem_fun( *this, &Toggle::PressToggle ) );
			}
			Toggle( int w, std::string lbl )
			: Widget( w, CONTROL_HEIGHT, lbl ), _pressed( false )
			{
				MouseUpSignal.connect( sigc::mem_fun( *this, &Toggle::PressToggle ) );
			}
			Toggle( std::string lbl )
			: Widget( ( lbl.size() + 1 ) * TEXT_WIDTH, CONTROL_HEIGHT, lbl ),
			  _pressed( false )
			{
				MouseUpSignal.connect( sigc::mem_fun( *this, &Toggle::PressToggle ) );
			}
				
			virtual ~Toggle()
			{}

			bool Value()
			{
				return _pressed;
			}
			void SetValue( bool v )
			{
				_pressed = v;
			}
			void PressToggle()
			{
				if( _pressed )
					_pressed = false;
				else
					_pressed = true;
			}
		protected:
			virtual void _draw();
		private:
			bool _pressed;
		};
	}
}
#endif // TOGGLE_H
