/* Copyright 2005 Nicholas Bishop
 * 
 * This file is part of SharpConstruct.
 *
 * SharpConstruct is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * SharpConstruct is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SharpConstruct; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#ifndef TOPWINDOW_H
#define TOPWINDOW_H

#include "Display.h"
#include "Group.h"
#include "Window.h"

namespace SharpConstruct
{
	namespace Interface
	{		
		class TopWindow : public Window< Group >
		{
		public:
			TopWindow( int w, int h, std::string lbl );
				
			virtual ~TopWindow();
		
			virtual void SetW( int w );
			virtual void SetH( int h );
			virtual void Draw();
		
			virtual void Run();
			virtual void DoEvents();
			virtual bool Close() = 0;
		
			virtual void Resize();
		
			virtual void MouseDown();
			virtual void MouseStillDown();
			virtual void MouseUp();
			virtual void DoubleClick();
			virtual void MouseMove();
			virtual void ScrollDown();
			virtual void ScrollUp();
			virtual void KeyDown();
			
			static inline TopWindow& Display() 
			{
				return *_display;
			}
			
			sigc::signal< void >& ResizeSignal();
		protected:
			Interface::Display* _device;
		private:
			static TopWindow* _display;
			Widget* _last_pressed_widget;
			sigc::signal< void > _resize_signal;
		};
	}
}
#endif // TOPWINDOW_H
