/* Copyright 2004, 2005 Nicholas Bishop
 * 
 * This file is part of SharpConstruct.
 *
 * SharpConstruct is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * SharpConstruct is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SharpConstruct; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#ifndef VIEW_H
#define VIEW_H

#include "ArcBall.h"
#include "Brush.h"
#include "Color.h"
#include "MeshHistory.h"
#include "Texture.h"
#include "Widget.h"
//#include <Magick++.h>

namespace SharpConstruct
{
	namespace Interface
	{
		class ViewGL : public Widget
		{
		public:
			ViewGL( int, int, int, int );
			virtual ~ViewGL();
		
			void ResetView();
			void Refresh();
			void SetBackgroundColor( Color );
			const Color& BackgroundColor() const;
			
			MeshHistory *_history;
			Brush *MainBrush;
		
			//void UpdateTexture( Magick::Image * );
			void DeactivateTexture();
		
			
			void ToggleSmoothed();
			void ToggleDoubleSided();
			
			//virtual void Draw() ; ///
		
			sigc::signal< void > ChangedMeshSignal;
		protected:
			virtual void _draw();
			virtual void _draw_interface();
		private:
			bool _rendered;
			bool _store_view;
			bool _display_copied_view;
		
			Math::Point2D< int > _last_mouse_location;
		
			bool _recalc_projected, _only_recalc_damaged;
			std::vector< Optimized::Point3D > _projected_points;
			std::vector< int > _damaged_area;
		
			// Texturing
			//Magick::Image *_texture;
		
			// Initialize OpenGL
			void _make_view_context();
			void _revert_view_context();
			void _orientate_view();
			void _initialize();
			void _simple_initialize();
			bool _first_run;
			bool _initialized;
			unsigned _texture_buffer;
		
			unsigned _vertex_buffer, _index_buffer;
		
			void _draw_mesh();
			bool _update_vertex_buffer;
			bool _update_index_buffer;
			void _find_projected_points();
			void _find_view_normals();
			
			SharpConstruct::Optimized::Point3D _location;
			float _cam_distance;
		
			double _projection_matrix[ 16 ];
			int _viewport[ 4 ];
			SharpConstruct::Optimized::Point3D _projected_cursor;
		
			// ArcBall
			ArcBall _arc_ball;
			
			Color _background_color;
			
			//
			void _mouse_down();
			void _mouse_up();
			void _mouse_move();
			void _scroll_down();
			void _scroll_up();
			
			bool _smoothed;
			bool _double_sided;
		};
	}
}

#endif // VIEW_H
