/* Copyright 2005 Nicholas Bishop
 * 
 * This file is part of SharpConstruct.
 *
 * SharpConstruct is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * SharpConstruct is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SharpConstruct; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#ifndef WIDGET_H
#define WIDGET_H

#include "Interface.h"
#include <sigc++/sigc++.h>
#include <string>

namespace SharpConstruct
{
	namespace Interface
	{
		class Widget : virtual public Drawable, virtual public Handler,
		               public Math::Rect2D< int >
		{
		public:
			enum Cursors
			{ DEFAULT = 68, CROSS = 34, HORIZ_ARROWS = 108, IBEAM = 152 };
		
			Widget( int x, int y, int w, int h, std::string lbl, int brd )
			: Math::Rect2D< int >( x, y, w, h ),
			  _enabled( true ), _label( lbl ), _border( brd ), _cursor( DEFAULT )
			{}
			Widget( int x, int y, int w, int h, std::string lbl )
			: Math::Rect2D< int >( x, y, w, h ),
			  _enabled( true ), _label( lbl ), _border( 0 ), _cursor( DEFAULT )
			{}
			Widget( int x, int y, int w, int h )
			: Math::Rect2D< int >( x, y, w, h ),
			  _enabled( true ), _border( 0 ), _cursor( DEFAULT )
			{}
			Widget( int w, int h, std::string lbl )
			: Math::Rect2D< int >( 0, 0, w, h ),
			  _enabled( true ), _label( lbl ), _border( 0 ), _cursor( DEFAULT )
			{}
			Widget( int w, int h )
			: Math::Rect2D< int >( 0, 0, w, h ),
			  _enabled( true ), _border( 0 ), _cursor( DEFAULT )
			{}
			Widget( std::string lbl, int brd )
			: Math::Rect2D< int >( 0, 0, 0, 0 ),
			  _enabled( true ), _label( lbl ), _border( brd ), _cursor( DEFAULT )
			{}	
			Widget( int brd )
			: Math::Rect2D< int >( 0, 0, 0, 0 ),
			  _enabled( true ), _border( brd ), _cursor( DEFAULT )
			{}
			Widget( std::string lbl )
			: Math::Rect2D< int >( 0, 0, 0, 0 ),
			  _enabled( true ), _label( lbl ), _border( 0 ), _cursor( DEFAULT )
			{}
			Widget()
			: Math::Rect2D< int >( 0, 0, 0, 0 ),
			  _enabled( true ), _border( 0 ), _cursor( DEFAULT )
			{}
			
			virtual ~Widget()
			{}
			
			virtual void Show();
			virtual void Hide();
			virtual void ToggleVisible();
				
			virtual void MouseDown();
			virtual void MouseStillDown();
			virtual void MouseUp();
			virtual void DoubleClick();
			virtual void MouseMove();
			virtual void ScrollDown();
			virtual void ScrollUp();
			virtual void KeyDown();
			virtual bool DoShortcut();
				
			virtual void AutoResize();
				
			//static Responders& GetResponders();
			//static ResponderWidgets Responders;

			static Widget* ActiveWidget();
			bool Active() const;
			void MakeActive();
			virtual void MakeInactive();
			static Widget* KeyboardFocus();
			bool Focused() const;
			void MakeFocused();
			void MakeUnfocused();
			bool Enabled() const;
			void SetEnabled( bool e );
			std::string Label() const;
			void SetLabel( std::string lbl );
			int Border() const;
			void SetBorder( int b );
			KeyData Shortcut() const;
			void SetShortcut( KeyData key );
			Cursors Cursor() const;
			void SetCursor( Cursors c );
			
			virtual bool Contains( const Math::Point2D< int >& in ) const;
			virtual bool Contains() const;
			
			// Note: most of the time, prefer modifying _draw() rather than Draw().
			//virtual void Draw();

			sigc::signal< void > MouseDownSignal;
			sigc::signal< void > MouseStillDownSignal;
			sigc::signal< void > MouseUpSignal;
			sigc::signal< void >& DoubleClickSignal();
			sigc::signal< void > MouseMoveSignal;
			sigc::signal< void > ScrollDownSignal;
			sigc::signal< void > ScrollUpSignal;
		private:
			static Widget* _active_widget;
			static Widget* _keyboard_focus;
			bool _enabled;
			std::string _label;
			int _border;
			KeyData _shortcut;
			Cursors _cursor;
		
			sigc::signal< void > _double_click_signal;
		};
	}
}
#endif // WIDGET_H
