/* Copyright 2005 Nicholas Bishop
 * 
 * This file is part of SharpConstruct.
 *
 * SharpConstruct is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * SharpConstruct is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SharpConstruct; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#ifndef XDISPLAYGL_H
#define XDISPLAYGL_H

#include "Display.h"
#include <GL/glx.h>
#include <X11/Xlib.h>
#include <X11/cursorfont.h>
#include <X11/extensions/xf86vmode.h>

namespace XLib
{
	typedef ::Display Display;
	typedef ::Window Window;
	typedef int Screen;
	typedef ::XSetWindowAttributes XSetWindowAttributes;
	typedef ::GLXContext GLXContext;
}

namespace SharpConstruct
{
	namespace Interface
	{
		class TopWindow;
		
		class XDisplayGL : public virtual Display
		{
		public:
			XDisplayGL( TopWindow& ctrl, int w, int h, std::string lbl );
				
			virtual ~XDisplayGL();
		
			virtual void Draw();
			virtual void Postwork();
		
			virtual void Run();
			virtual void DoEvents();
		
			virtual void Close();
		
			virtual int W() const;
			virtual int H() const;
			virtual void SetSize( const int w, const int h );
			std::string Label() const;
			virtual void SetLabel( const std::string& lbl );

			virtual void SetCursor( Widget::Cursors c );
		
			static XLib::Display* GetXDisplay()
			{
				return _xdisplay;
			}
		private:
			void _initialize( int w, int h );
			void _make_current();
		
			TopWindow& _controller;
		
			static XLib::Display* _xdisplay;
			XLib::Window _window;
			XLib::Screen _screen;
			XLib::XSetWindowAttributes _attributes;
			XLib::GLXContext _glx_context;

			std::string _label;
			Widget::Cursors _cursor;
		
			bool _check_double_click;
		};
	}
}
#endif // XDISPLAYGL_H
