/* Copyright 2004, 2005 Nicholas Bishop
 * 
 * This file is part of SharpConstruct.
 *
 * SharpConstruct is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * SharpConstruct is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SharpConstruct; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#include "Brush.h"

using namespace SharpConstruct;

Brush::Brush()
: _modeling_mode( DISPLACE ), _position_edit( true ), _color_edit( false ),
  _radius( 10 ), _strength( -0.025 ), _intensity( 0.025 ),
  _color( 1.0, 1.0, 1.0 ),
  _symmetry_x( false ), _symmetry_y( false ), _symmetry_z( false )
{}

Brush::~Brush()
{}
	
void Brush::SetModelingMode( ModelingMode input )
{
	_modeling_mode = input;
}

Brush::ModelingMode Brush::GetModelingMode() const
{
	return _modeling_mode;
}

void Brush::SetPositionEdit( bool input )
{
	_position_edit = input;
}

bool Brush::GetPositionEdit() const
{
	return _position_edit;
}

void Brush::SetColorEdit( bool input )
{
	_color_edit = input;
}

bool Brush::GetColorEdit() const
{
	return _color_edit;
}

void Brush::SetRadius( float input )
{
	_radius = input;
}

void Brush::AdjustRadius( float diameter )
{
	_radius = ( diameter * _radius ) / 100;
}

float Brush::GetRadius() const
{
	return _radius;
}

void Brush::SetStrength( float input )
{
	_strength = input;
}

float Brush::GetStrength() const
{
	return _strength;
}

void Brush::SetIntensity( float input )
{
	_intensity = input;
}

float Brush::GetIntensity() const
{
	return _intensity;
}

void Brush::SetRed( float input )
{
	_color.Red = input;
	
	if( _color.Red < 0 )
		_color.Red = 0;
	else if( _color.Red > 1 )
		_color.Red = 1;
}

float Brush::GetRed() const
{
	return _color.Red;
}

void Brush::SetGreen( float input )
{
	_color.Green = input;
	
	if( _color.Green  < 0 )
		_color.Green  = 0;
	else if( _color.Green  > 1 )
		_color.Green  = 1;
}

float Brush::GetGreen() const
{
	return _color.Green;
}

void Brush::SetBlue( float input )
{
	_color.Blue = input;
	
	if( _color.Blue < 0 )
		_color.Blue = 0;
	else if( _color.Blue > 1 )
		_color.Blue = 1;
}

float Brush::GetBlue() const
{
	return _color.Blue;
}

void Brush::SetSymmetryX( bool input )
{
	_symmetry_x = input;
}

bool Brush::GetSymmetryX() const
{
	return _symmetry_x;
}

void Brush::SetSymmetryY( bool input )
{
	_symmetry_y = input;
}

bool Brush::GetSymmetryY() const
{
	return _symmetry_y;
}

void Brush::SetSymmetryZ( bool input )
{
	_symmetry_z = input;
}

bool Brush::GetSymmetryZ() const
{
	return _symmetry_z;
}
