/* Copyright 2005 Nicholas Bishop
 * 
 * This file is part of SharpConstruct.
 *
 * SharpConstruct is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * SharpConstruct is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SharpConstruct; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#include "Divider.h"
#include <GL/gl.h>

using namespace SharpConstruct::Interface;

void VerticalDivider::_draw()
{
	// Draw border
	if( Contains() )
		glColor4f( 0, 0, 0, 1 );
	else
		glColor4f( 0.2, 0.2, 0.2, 1 );
	glRecti( X(), Y(), Right() + 1, Bottom() + 1 );
	
	// Fill inside
	if( Contains() )
		glColor4f( 0.6, 0.6, 0.6, 1 );
	else
		glColor4f( 0.5, 0.5, 0.5, 1 );
	glRecti( X() + 1, Y() + 1, Right(), Bottom() );
	
	// Draw grip
	const int hw = W() / 2, hh = H() / 2;
	glColor4f( 0.8, 0.8, 0.8, 1 );
	glRecti( X() + hw - 1, Y() + hh - 11, X() + hw + 1, Y() + hh - 9 );
	glRecti( X() + hw - 1, Y() + hh - 6, X() + hw + 1, Y() + hh - 4 );
	glRecti( X() + hw - 1, Y() + hh - 1, X() + hw + 1, Y() + hh + 1 );
	glRecti( X() + hw - 1, Y() + hh + 4, X() + hw + 1, Y() + hh + 6 );
	glRecti( X() + hw - 1, Y() + hh + 9, X() + hw + 1, Y() + hh + 11 );
}

void VerticalDivider::_mouse_move()
{	
	if( Event::Mouse.LeftButtonDown )
	{
		SetX( Event::Mouse.Location.X() - 4 );
		if( X() < _left_bound )
			SetX( _left_bound );
		else if( Right() > _right_bound )
			SetRight( _right_bound );
		ResizeSignal.emit();
	}
}
