/* Copyright 2005 Nicholas Bishop
 * 
 * This file is part of SharpConstruct.
 *
 * SharpConstruct is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * SharpConstruct is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SharpConstruct; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#include "InputDialog.h"
#include "Spacer.h"
#include "Text.h"
#include "TopWindow.h"
#include <GL/gl.h>

#include <iostream>

using namespace SharpConstruct::Interface;

InputDialog::InputDialog( const std::string& lbl,
                          const std::vector< std::string >& button_labels  )
: Dialog( lbl ), _controls( true, 4, 4 )
{
	SetX( 30 );
	SetY( 30 );
	SetW( 350 );
	
	// Why the fuck do we need two seperate loops for this to run???
	for( unsigned i = 0; i < button_labels.size(); i++ )
		_buttons.push_back( Button( button_labels[ i ] ) );
	for( unsigned i = 0; i < button_labels.size(); i++ )
		_controls.PushBack( _buttons[ i ] );
	
	PushBack( _dir );
	PushBack( _controls );
	
	_dir.MouseDownSignal();
}

void InputDialog::_draw()
{
	Dialog::_draw();
	// Darken background to indicate this is a modal window
	glColor4f( 0, 0, 0, 0.3 );
	glRecti( 0, 0, TopWindow::Display().W(), TopWindow::Display().H() );
	
	Dialog::_draw();
}

void InputDialog::_button_catcher()
{
	_selected_button = ActiveWidget()->Label();
	Hide();
}

std::string InputDialog::ModalInput( Group& p,
                                     const std::string& lbl,
                                     const std::string& b1,
                                     const std::string& b2 )
{
	std::vector< std::string > tmp;
	tmp.push_back( b1 );
	tmp.push_back( b2 );
	InputDialog d( lbl, tmp );
	d.GetButtons()[ 0 ].MouseUpSignal.connect( sigc::mem_fun( d, &InputDialog::_button_catcher ) );
	d.GetButtons()[ 1 ].MouseUpSignal.connect( sigc::mem_fun( d, &InputDialog::_button_catcher ) );
	d._dir.ReturnSignal().connect( sigc::mem_fun( d, &InputDialog::_button_catcher ) );
	Spacer blank( 0, 0, TopWindow::Display().W(), TopWindow::Display().H() );
	p.PushBack( blank );
	p.PushBack( d );
	while( d.Visible() )
	{
		TopWindow::Display().DoEvents();
		Widget::ResetOrderCount();
		TopWindow::TopWindow::Display().Draw();
	}
	
	p.Erase( p.Find( &blank ) );
	p.Erase( p.Find( &d ) );
	
	if( d._selected_button == b2 )
		return "";
	else
		return d._dir.Label();
}
