/* Copyright 2004, 2005 Nicholas Bishop
 * 
 * This file is part of SharpConstruct.
 *
 * SharpConstruct is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * SharpConstruct is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SharpConstruct; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

//#include "Triangle3D.h"

#include "Mesh.h"
#include "Color.h"
#include <cmath>

#include <iostream>

using SharpConstruct::Optimized::Point3D;
using SharpConstruct::Optimized::Normal3D;

void Mesh::DisplaceAreas()
{
	_modified_vertex_indices.clear();
	_lowest_index = _vertex_locations.size() - 1;
	_highest_index = 0;

	_displace_area( _current_selection, 0 );

	//std::cout << MainBrush << std::endl;
	
	if( MainBrush->GetSymmetryX() )
		_displace_area( _get_mirror_point( true, false, false ), 1 );
	if( MainBrush->GetSymmetryY() )
		_displace_area( _get_mirror_point( false, true, false ), 2 );
	if( MainBrush->GetSymmetryZ() )
		_displace_area( _get_mirror_point( false, false, true ), 3 );
	if( MainBrush->GetSymmetryX() && MainBrush->GetSymmetryY() )
		_displace_area( _get_mirror_point( true, true, false ), 4 );
	if( MainBrush->GetSymmetryY() && MainBrush->GetSymmetryZ() )
		_displace_area( _get_mirror_point( false, true, true ), 5 );
	if( MainBrush->GetSymmetryX() && MainBrush->GetSymmetryZ() )
		_displace_area( _get_mirror_point( true, false, true ), 6 );
	if( MainBrush->GetSymmetryX() && MainBrush->GetSymmetryY() && MainBrush->GetSymmetryZ() )
		_displace_area( _get_mirror_point( true, true, true ), 7 );

	if( MainBrush->GetPositionEdit() )
		_recalculate_modified_normals();
	
	_change_operation();
}

void Mesh::StartGrab()
{
	if( MainBrush->GetPositionEdit() )
	{
		_lowest_index = _vertex_locations.size() - 1;
		_highest_index = 0;
		
		_find_active_vertices( _grab_vertices[ 0 ], _grab_vertex_distances[ 0 ],
							   _current_selection, 0 );
		
			
		
		if( MainBrush->GetSymmetryX() )
			_find_active_vertices( _grab_vertices[ 1 ], _grab_vertex_distances[ 1 ],
								   _get_mirror_point( true, false, false ), 1 );
		if( MainBrush->GetSymmetryY() )
			_find_active_vertices( _grab_vertices[ 2 ], _grab_vertex_distances[ 2 ],
								   _get_mirror_point( false, true, false ), 2 );
		if( MainBrush->GetSymmetryZ() )
			_find_active_vertices( _grab_vertices[ 3 ], _grab_vertex_distances[ 3 ],
								   _get_mirror_point( false, false, true ), 3 );
		if( MainBrush->GetSymmetryX() && MainBrush->GetSymmetryY() )
			_find_active_vertices( _grab_vertices[ 4 ], _grab_vertex_distances[ 4 ],
								   _get_mirror_point( true, true, false ), 4 );
		if( MainBrush->GetSymmetryY() && MainBrush->GetSymmetryZ() )
			_find_active_vertices( _grab_vertices[ 5 ], _grab_vertex_distances[ 5 ],
								   _get_mirror_point( false, true, true ), 5 );
		if( MainBrush->GetSymmetryX() && MainBrush->GetSymmetryZ() )
			_find_active_vertices( _grab_vertices[ 6 ], _grab_vertex_distances[ 6 ],
								   _get_mirror_point( true, false, true ), 6 );
		if( MainBrush->GetSymmetryX() && MainBrush->GetSymmetryY() && MainBrush->GetSymmetryZ() )
			_find_active_vertices( _grab_vertices[ 7 ], _grab_vertex_distances[ 7 ],
								   _get_mirror_point( true, true, true ), 7 );
	}
}

void Mesh::ContinueGrab( float delta_x, float delta_y )
{
	if( MainBrush->GetPositionEdit() )
	{
		Normal3D drag;
		drag.Zero();
		drag += _view_right_normal * delta_x;
		drag += _view_up_normal * delta_y;
		
		_continue_grab( _grab_vertices[ 0 ], _grab_vertex_distances[ 0 ], drag );
		
		if( MainBrush->GetSymmetryX() )
			_continue_grab( _grab_vertices[ 1 ], _grab_vertex_distances[ 1 ],
							Normal3D( -drag.X(), drag.Y(), drag.Z() ) );
		if( MainBrush->GetSymmetryY() )
			_continue_grab( _grab_vertices[ 2 ], _grab_vertex_distances[ 2 ],
							Normal3D( drag.X(), -drag.Y(), drag.Z() ) );
		if( MainBrush->GetSymmetryZ() )
			_continue_grab( _grab_vertices[ 3 ], _grab_vertex_distances[ 3 ],
							Normal3D( drag.X(), drag.Y(), -drag.Z() ) );
		if( MainBrush->GetSymmetryX() && MainBrush->GetSymmetryY() )
			_continue_grab( _grab_vertices[ 4 ], _grab_vertex_distances[ 4 ],
							Normal3D( -drag.X(), -drag.Y(), drag.Z() ) );
		if( MainBrush->GetSymmetryY() && MainBrush->GetSymmetryZ() )
			_continue_grab( _grab_vertices[ 5 ], _grab_vertex_distances[ 5 ],
							Normal3D( drag.X(), -drag.Y(), -drag.Z() ) );
		if( MainBrush->GetSymmetryX() && MainBrush->GetSymmetryZ() )
			_continue_grab( _grab_vertices[ 6 ], _grab_vertex_distances[ 6 ],
							Normal3D( -drag.X(), drag.Y(), -drag.Z() ) );
		if( MainBrush->GetSymmetryX() && MainBrush->GetSymmetryY() && MainBrush->GetSymmetryZ() )
			_continue_grab( _grab_vertices[ 7 ], _grab_vertex_distances[ 7 ],
							Normal3D( -drag.X(), -drag.Y(), -drag.Z() ) );

		_recalculate_modified_normals();
	}
}

void Mesh::EndGrab()
{
	for( int i = 0; i < 8; i++ )
	{
		_grab_vertices[ i ].clear();
		_grab_vertex_distances[ i ].clear();
	}
	
	_change_operation();
}

void Mesh::SmoothAreas()
{
	_lowest_index = _vertex_locations.size() - 1;
	_highest_index = 0;
	
	_smooth_area( _current_selection, 0 );

	if( MainBrush->GetSymmetryX() )
		_smooth_area( _get_mirror_point( true, false, false ), 1 );
	if( MainBrush->GetSymmetryY() )
		_smooth_area( _get_mirror_point( false, true, false ), 2 );
	if( MainBrush->GetSymmetryZ() )
		_smooth_area( _get_mirror_point( false, false, true ), 3 );
	if( MainBrush->GetSymmetryX() && MainBrush->GetSymmetryY() )
		_smooth_area( _get_mirror_point( true, true, false ), 4 );
	if( MainBrush->GetSymmetryY() && MainBrush->GetSymmetryZ() )
		_smooth_area( _get_mirror_point( false, true, true ), 5 );
	if( MainBrush->GetSymmetryX() && MainBrush->GetSymmetryZ() )
		_smooth_area( _get_mirror_point( true, false, true ), 6 );
	if( MainBrush->GetSymmetryX() && MainBrush->GetSymmetryX() && MainBrush->GetSymmetryZ() )
		_smooth_area( _get_mirror_point( true, true, true ), 7 );

	if( MainBrush->GetPositionEdit() )
		_recalculate_modified_normals();
	_modified_vertex_indices.clear();
	
	_change_operation();
}

void Mesh::MaskAreas()
{}

void Mesh::_displace_area( Point3D area_center, int mirror_index )
{
	_find_active_vertices( _active_vertex_indices, _active_vertex_distances,
	                       area_center, mirror_index );

	if( MainBrush->GetPositionEdit() )
	{
		Point3D area_normal = _calculate_area_normal();
		for( unsigned i = 0; i < _active_vertex_indices.size(); i++ )
		{
			//_vertex_colors[ _active_vertex_indices[ i ] ].Red = 0;
			_vertex_locations[ _active_vertex_indices[ i ] ].X() += area_normal.X() *
																 ( ( 1 - pow( 
																 _active_vertex_distances[ i ] /
																 MainBrush->GetRadius(), 2 ) ) *
																 MainBrush->GetStrength() );
			_vertex_locations[ _active_vertex_indices[ i ] ].Y() += area_normal.Y() *
																 ( ( 1 - pow( 
																 _active_vertex_distances[ i ] /
																 MainBrush->GetRadius(), 2 ) ) *
																 MainBrush->GetStrength() );
			_vertex_locations[ _active_vertex_indices[ i ] ].Z() += area_normal.Z() *
																 ( ( 1 - pow( 
																 _active_vertex_distances[ i ] /
																 MainBrush->GetRadius(), 2 ) ) *
																 MainBrush->GetStrength() );
		}
	}
	/*if( MainBrush->GetColorEdit() && _active_vertex_indices.size() && _texture != NULL )
	{
		// THIS IS SO UGLY!!!
		
		int i, j, center = _active_vertex_indices[ 0 ], vertex, polygon;
		float v2dis = 99999;
		
		for( i = 1; i < _active_vertex_distances.size(); i++ )
		{
			if( _active_vertex_distances[ i ] < _active_vertex_distances[ center ] )
				center = i;
		}
		
		// Loop through each connected polygon
		for( i = 0; i < _vertex_polygon_indices[ center ].size(); i++ )
		{
			polygon = _vertex_polygon_indices[ center ][ i ];
			
			float u, v;
			float ratios[ 3 ];
			Point3D p[ 3 ];
			for( j = 0; j < 3; j++ )
				p[ j ] = _vertices[ _indices[ polygon ] ].Location;
			
			//std::cout << p[ 2 ].X() << ", " << p[ 2 ].Y() << ", " << p[ 2 ].Z() << std::endl;
			Barycentric< float > b( &p[ 0 ].X(), &p[ 1 ].X(), &p[ 2 ].X() );
			//std::cout << b.Eval( &_current_selection.X(), ratios ) << std::endl;
			for( j = 0; j < 3; j++ )
				std::cout << ratios[ j ] << ", "; std::cout << std::endl;
			
			if( ratios[ 0 ] > 0 && ratios[ 1 ] > 0 && ratios[ 2 ] > 0 )
			{
				//std::cout << polygon << std::endl;
				for( j = 0; j < 3; j++ )
				{
					u += ratios[ j ] * _vertices[ _indices[ polygon * 3 + j ] ].U;
					v += ratios[ j ] * _vertices[ _indices[ polygon * 3 + j ] ].V;
				}
				
				if( u < 0 )
					u = 0;
				if( v < 0 )
					v = 0;
				if( u > 1 )
					u = 1;
				if( v > 1 )
					v = 1;
				
				_texture->pixelColor( u * _texture->columns(), v * _texture->rows(),
								  Magick::Color( "red" ) );
			}
			
			 // Loop through each vertex in the current polygon
			for( j = 0; j < 3; j++ )
			{
				vertex = _indices[ _vertex_polygon_indices[ center ][ i ] * 3 + j ];
				if( vertex != center )
				{
					float tmp = _vertices[ vertex ].Location.Distance( _current_selection );
					
					if( tmp < v2dis )
					{
						v2dis = tmp;
						polygon = _vertex_polygon_indices[ center ][ i ];
					}
				}					
			}
		}*/
		
		
		//ratios[ 0 ] = dist( v, edge(v2,v3) ) / dist( v1, edge(v2,v3) )
		//ratios[ 1 ] = dist( v, edge(v3,v1) ) / dist( v2, edge(v3,v1) )
		//ratios[ 2 ] = dist( v, edge(v1,v2) ) / dist( v3, edge(v1,v2) )
		
		/*failed int first, last;
		Point3D p[ 3 ]; // The three points in the polygon
		float ratios[ 3 ], total = 0, u = 0, v = 0;
		for( i = 0; i < 3; i++ )
			p[ i ] = _vertices[ _indices[ polygon * 3 + i ] ].Location;
		
		for( i = 0; i < 3; i++ )
			total += _current_selection.DistanceFrom( p[ i ] );
		
		std::cout << "t: " << total;
		
		for( i = 0; i < 3; i++ )
		{
			first = i + 1;
			last = first + 1;
			if( first == 2 )
				last = 0;
			if( first == 3 )
			{
				first = 0;
				last = 1;
			}
			
			ratios[ i ] = _current_selection.DistanceFrom( ( p[ first ] +
			                                                 p[ last ] ) / 2 ) / total;
			std::cout << ", " << ratios[ i ];
		}
		std::cout << std::endl;*/
		
		/*Another failed attempt		
		for( i = 0; i < 3; i++ )
		{
			first = i + 1;
			last = first + 1;
			if( first == 2 )
				last = 0;
			if( first == 3 )
			{
				first = 0;
				last = 1;
			}
			denom = p[ i ].DistanceFrom( ( p[ first ] + p[ last ] ) / 2 );
			if( denom != 0 )
				ratios[ i ] = 1 - ( p[ i ].DistanceFrom( _current_selection ) / denom );
			else
				ratios[ i ] = 0;
			                    
			std::cout << ratios[ i ] << ", ";
		}
		std::cout << ratios[ 0 ] + ratios[ 1 ] + ratios[ 2 ] << std::endl;*/


		
		/*// The three 'f' distances are the normalized dists from each vertex to the cursel
		float f_dists[ 3 ];
		float v_dists[ 3 ];
		float total = 0;
		for( i = 0; i < 3; i++ )
		{
			f_dists[ i ] = _current_selection.DistanceToPoint(
				_vertices[ _indices[ polygon * 3 + i ] ].Location );
			v_dists[ i ] = polygon_center.DistanceToPoint(
				_vertices[ _indices[ polygon * 3 + i ] ].Location );
			f_dists[ i ] = v_dists[ i ] - f_dists[ i ];
			if( f_dists[ i ] < 0 )
				f_dists[ i ] = 0;
			
			total += f_dists[ i ];			
		}
		
		for( i = 0; i < 3; i++ )
			f_dists[ i ] /= total;
		
		float u = 0, v = 0;
		for( i = 0; i < 3; i++ )
		{
			u += f_dists[ i ] * _vertices[ _indices[ polygon * 3 + i ] ].U;
			v += f_dists[ i ] * _vertices[ _indices[ polygon * 3 + i ] ].V;
		}*/
		//std::cout << "u: " << u << ", v: " << v << " | ";
		//std::cout<<u * _texture->columns()<<", " << v * _texture->rows() << std::endl;


		/*//int polygon = _active_vertex_indices[ 0 ];
		int i, x, y, center = _active_vertex_distances[ 0 ];
		
		for( i = 1; i < _active_vertex_distances.size(); i++ )
		{
			if( _active_vertex_distances[ i ] < _active_vertex_distances[ center ] )
				center = i;
		}
		
		x = _vertices[ _active_vertex_indices[ center ] ].U * _texture->columns();
		y = _vertices[ _active_vertex_indices[ center ] ].V * _texture->rows();
			
		_texture->pixelColor( x, y, Magick::Color( "red" ) );*/
	if( MainBrush->GetColorEdit() )
	{
		float intensity;
		SharpConstruct::Color* v;

		for( unsigned i = 0; i < _active_vertex_indices.size(); i++ )
		{
			v = &_vertex_colors[ _active_vertex_indices[ i ] ];
			intensity = ( 1 - pow( _active_vertex_distances[ i ] / MainBrush->GetRadius(), 2 ) ) *
					   MainBrush->GetIntensity();
			v->Red = v->Red + ( MainBrush->GetRed() - v->Red ) * intensity;
			v->Green = v->Green + ( MainBrush->GetGreen() - v->Green ) * intensity;
			v->Blue = v->Blue + ( MainBrush->GetBlue() - v->Blue ) * intensity;
		}
	}
}

void Mesh::_smooth_area( Point3D area_center, int mirror_index )
{
	int i, j, k, total, polygon_index;
	int previous_point_index, next_point_index;
	float strength, intensity;
	Point3D average;
	SharpConstruct::Color average_color;
	
	_find_active_vertices( _active_vertex_indices, _active_vertex_distances,
	                       area_center, mirror_index );

	// Loop through each active point
	for( i = 0; i < _active_vertex_indices.size(); i++ )
	{
		total = 0;
		average.Zero();
		average_color.SetToBlack();

		// Loop through each polygon using the current point
		for( j = 0; j < _vertex_polygon_indices[ _active_vertex_indices[ i ] ].size(); j++ )
		{
			// Set the current polygon
			polygon_index =_vertex_polygon_indices[ _active_vertex_indices[ i ] ][ j ];
			// Loop through each vertex in the polygon
			for( k = 0; k < 3; k++ )
				// If the current active point equals the current VertexIndex...
				if( _indices[ polygon_index * 3 + k ] == _active_vertex_indices[ i ] )
				{
					previous_point_index = k - 1;
					next_point_index = k + 1;
					if( previous_point_index < 0 )
						previous_point_index = 2;
					if( next_point_index == 3 )
						next_point_index = 0;
					
					average += _vertex_locations[ _active_vertex_indices[ i ] ];
					average += _vertex_locations[ _indices[ polygon_index * 3 + previous_point_index ] ];
					average += _vertex_locations[ _indices[ polygon_index * 3 + next_point_index ] ];
					
					average_color.Red += _vertex_colors[ _active_vertex_indices[ i ] ].Red;
					average_color.Green += _vertex_colors[ _active_vertex_indices[ i ] ].Green;
					average_color.Blue += _vertex_colors[ _active_vertex_indices[ i ] ].Blue;
					average_color.Red += _vertex_colors[ _indices[ polygon_index * 3 + previous_point_index ] ].Red;
					average_color.Green += _vertex_colors[ _indices[ polygon_index * 3 + previous_point_index ] ].Green;
					average_color.Blue += _vertex_colors[ _indices[ polygon_index * 3 + previous_point_index ] ].Blue;
					average_color.Red += _vertex_colors[ _indices[ polygon_index * 3 + next_point_index ] ].Red;
					average_color.Green += _vertex_colors[ _indices[ polygon_index * 3 + next_point_index ] ].Green;
					average_color.Blue += _vertex_colors[ _indices[ polygon_index * 3 + next_point_index ] ].Blue;
					
					total += 3;
				}
		}
		average /= total;
		average_color.Red /= total;
		average_color.Green /= total;
		average_color.Blue /= total;
		if( MainBrush->GetPositionEdit() )
		{
			strength = ( 1 - pow( _active_vertex_distances[ i ] / MainBrush->GetRadius(), 2 ) ) *
					   fabs( MainBrush->GetStrength() * 20 );
			_vertex_locations[ _active_vertex_indices[ i ] ] += ( average - _vertex_locations[ _active_vertex_indices[ i ] ] ) *
										strength;
		}
		if( MainBrush->GetColorEdit() )
		{
			SharpConstruct::Color* v = &_vertex_colors[ _active_vertex_indices[ i ] ];
			
			intensity = ( 1 - pow( 1 - _active_vertex_distances[ i ] /
			                  MainBrush->GetRadius(), 2 ) ) *
			            MainBrush->GetIntensity();
			v->Red = v->Red + ( average_color.Red - v->Red ) *
										intensity;
			v->Green = v->Green + ( average_color.Green - v->Green ) *
										intensity;
			v->Blue = v->Blue + ( average_color.Blue - v->Blue ) *
										intensity;
		}
	}
}

void Mesh::_continue_grab( std::vector< int > &vertex_indices,
                           std::vector< float > &vertex_distances,
                           Normal3D delta )
{
	for( unsigned i = 0; i < vertex_indices.size(); i++ )
	{
		_vertex_locations[ vertex_indices[ i ] ] += delta * ( 1 - pow( 
		                                   vertex_distances[ i ] /
	                                       MainBrush->GetRadius(), 2 ) );
	}
}

void Mesh::_mask_areas( Point3D area_center, int mirror_index )
{
	/*_find_active_vertices( _active_vertex_indices, _active_vertex_distances,
	                       area_center, mirror_index );
	
	int i;
	
	float intensity;
	Vertex3D *v;

	for( i = 0; i < _active_vertex_indices.size(); i++ )
	{
		v = &_vertices[ _active_vertex_indices[ i ] ];
		intensity = ( 1 - pow( _active_vertex_distances[ i ] / MainBrush->GetRadius(), 2 ) ) *
				   MainBrush->GetIntensity();
		v->Colors.MaskIntensity( v->Colors.MaskIntensity() +
		                     ( 0.25 - v->Colors.MaskIntensity() ) * intensity );
	}*/
}

Point3D Mesh::_get_mirror_point( bool x, bool y, bool z )
{
	Point3D mirror_point;
	mirror_point = _current_selection;

	if( x )
		mirror_point.X() = -mirror_point.X();
	if( y )
		mirror_point.Y() = -mirror_point.Y();
	if( z )
		mirror_point.Z() = -mirror_point.Z();
			
	return mirror_point;
}
