/* Copyright 2004, 2005 Nicholas Bishop
 * 
 * This file is part of SharpConstruct.
 *
 * SharpConstruct is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * SharpConstruct is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SharpConstruct; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#include "MeshHistory.h"

MeshHistory::MeshHistory( unsigned int max )
: _maximum( max ), _current_index( 0 ), _current_maximum_index( 0 )
{
	_meshes.resize( 1 );
	_meshes[ 0 ].Changed().connect( Changed().make_slot() );
}

MeshHistory::~MeshHistory()
{}

void MeshHistory::AddMesh()
{	
	_meshes[ _current_index ].ClearExtraData();
	if( _current_index == _maximum - 1 )
	{
		_meshes.erase( _meshes.begin() );
		_current_index--;
		_current_maximum_index--;
	}
	_meshes.resize( _current_index + 2 );
	_meshes[ _current_index + 1 ].CopyEssentials( _meshes[ _current_index ] );
	if( _current_index == _current_maximum_index )
		_current_maximum_index++;
	else
		_current_maximum_index = _current_index + 1;
	_current_index++;
	_meshes[ _current_index ].Changed().slots().erase(
		_meshes[ _current_index ].Changed().slots().begin(),
		_meshes[ _current_index ].Changed().slots().end() );
	_meshes[ _current_index ].Changed().connect( Changed().make_slot() );
	Changed()();
}

void MeshHistory::Undo()
{
	_current_index--;
	if( _current_index < 0 )
		_current_index = 0;
	Changed()();
}

void MeshHistory::Redo()
{
	_current_index++;
	if( _current_index > _current_maximum_index )
		_current_index = _current_maximum_index;
	Changed()();
}

void MeshHistory::Clear()
{
	Mesh temp = GetCurrentMesh();
	_current_index = 0;
	_current_maximum_index = 0;
	_meshes.clear();
	_meshes.push_back( temp );
	Changed()();
}

Mesh& MeshHistory::GetCurrentMesh()
{
	return _meshes[ _current_index ];
	Changed()();
}

int MeshHistory::Maximum() const
{
	return _maximum;
}
void MeshHistory::SetMaximum( unsigned int max )
{
	_maximum = max;
	
	if( max < _meshes.size() )
	{
		_meshes.erase( _meshes.begin(), _meshes.begin() + ( _meshes.size() - max - 1 ) );
		_current_maximum_index = max;
		_current_index = max - 1;
	}
	
	Changed()();
}

bool MeshHistory::UndoPossible() const
{
	if( _current_index > 0 )
		return true;
	return false;	
}

bool MeshHistory::RedoPossible() const
{
	if( _current_index < _current_maximum_index )
		return true;
	return false;	
}

sigc::signal< void >& MeshHistory::Changed()
{
	return _changed;
}
