/* Copyright 2005 Nicholas Bishop
 * 
 * This file is part of SharpConstruct.
 *
 * SharpConstruct is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * SharpConstruct is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SharpConstruct; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#include "MessageDialog.h"
#include "Spacer.h"
#include "Text.h"
#include "TopWindow.h"
#include <GL/gl.h>

using namespace SharpConstruct::Interface;

MessageDialog::MessageDialog( const std::string& lbl, const std::string& message,
                              const std::vector< std::string >& button_labels  )
: Dialog( lbl ), _controls( true, 4, 4 )
{
	_text.SetSizeManaged( false );
	_text.Set( 0, 0, 500, 100 );
	_text.SetLabel( message );
	
	// Why the fuck do we need two seperate loops for this to run???
	for( unsigned i = 0; i < button_labels.size(); i++ )
		_buttons.push_back( Button( button_labels[ i ] ) );
	for( unsigned i = 0; i < button_labels.size(); i++ )
		_controls.PushBack( _buttons[ i ] );
	
	PushBack( _text );
	PushBack( _controls );

	SetX( 30 );
	SetY( 30 );
}

void MessageDialog::_draw()
{
	Dialog::_draw();
	// Darken background to indicate this is a modal window
	glColor4f( 0, 0, 0, 0.3 );
	glRecti( 0, 0, TopWindow::Display().W(), TopWindow::Display().H() );
	
	Dialog::_draw();
	
	int x = _text.X() + 4, y = _text.Y() + 10;
	
	GLOut::glout << GLFormat::Align::Left();
	for( int i = 0; i < _text.Label().length(); i++ )
	{
		if( _text.Label().substr( i, 1 ) == "\n" )
		{
			x = _text.X() + 4;
			y += TEXT_HEIGHT;
		}
		else
		{
			GLOut::glout << GLFormat::Location( x, y )
						 << _text.Label().substr( i, 1 );
			x += TEXT_WIDTH;
			if( x + TEXT_WIDTH > _text.Right() )
			{
				x = _text.X() + 4;
				y += TEXT_HEIGHT;
			}
		}
	}
}

void MessageDialog::_button_catcher()
{
	_selected_button = ActiveWidget()->Label();
	Hide();
}

std::string MessageDialog::ModalMessage( Group& p,
                                         const std::string& lbl,
                                         const std::string& message,
                                         const std::vector< std::string >& btns )
{
	MessageDialog d( lbl, message, btns );
	for( unsigned i = 0; i < d.GetButtons().size(); i++ )
		d.GetButtons()[ i ].MouseUpSignal.connect( sigc::mem_fun( d, &MessageDialog::_button_catcher ) );
	Spacer blank( 0, 0, TopWindow::Display().W(), TopWindow::Display().H() );
	p.PushBack( blank );
	p.PushBack( d );
	while( d.Visible() )
	{
		TopWindow::Display().DoEvents();
		Widget::ResetOrderCount();
		TopWindow::Display().Draw();
	}
	
	p.Erase( p.Find( &blank ) );
	p.Erase( p.Find( &d ) );
	
	return d._selected_button;
}

std::string MessageDialog::ModalMessage( Group& p,
                                         const std::string& lbl,
                                         const std::string& message,
                                         const std::string& b1,
                                         const std::string& b2 )
{
	std::vector< std::string > tmp;
	tmp.push_back( b1 );
	tmp.push_back( b2 );
	return ModalMessage( p, lbl, message, tmp );
}
std::string MessageDialog::ModalMessage( Group& p,
                                         const std::string& lbl,
                                         const std::string& message,
                                         const std::string& b1 )
{
	std::vector< std::string > tmp;
	tmp.push_back( b1 );
	return ModalMessage( p, lbl, message, tmp );
}
std::string MessageDialog::ModalMessage( Group& p,
                                         const std::string& lbl,
                                         const std::string& message,
                                         const std::string& b1,
                                         const std::string& b2,
                                         const std::string& b3 )
{
	std::vector< std::string > tmp;
	tmp.push_back( b1 );
	tmp.push_back( b2 );
	tmp.push_back( b3 );
	return ModalMessage( p, lbl, message, tmp );
}
