/* Copyright 2005 Nicholas Bishop
 * 
 * This file is part of SharpConstruct.
 *
 * SharpConstruct is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * SharpConstruct is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SharpConstruct; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#include "MainWindow.h"
#include "NewDialog.h"

using namespace SharpConstruct::Interface;

NewDialog::NewDialog( MainWindow& mw )
: Dialog( "New model" ),
  _parent( mw ),
  _type( false, 0, 4 ),
  _plane( "Plane" ),
  _sphere( "Sphere" ),
  _plane_div_h( "HDiv" ),
  _plane_div_v( "VDiv" ),
  _sphere_div( "Div" ),
  _create( "Create" )
{
	SetY( 88 );
	
	_plane.MouseUpSignal.connect( sigc::mem_fun( *this, &NewDialog::_set_plane ) );
	_plane.SetValue( true );
	_sphere.MouseUpSignal.connect( sigc::mem_fun( *this, &NewDialog::_set_sphere ) );
	
	_plane_div_h.SetProperties( 1, 200, 1 );
	_plane_div_h.Value( 50 );
	_plane_div_v.SetProperties( 1, 200, 1 );
	_plane_div_v.Value( 50 );
	_sphere_div.SetProperties( 1, 10, 1 );
	_sphere_div.Value( 4 );
	
	_plane.SetW( 100 );
	_sphere.SetW( 100 );
	_create.SetW( 100 );
	_type.PushBack( _plane );
	_type.PushBack( _sphere );
	
	_create.MouseUpSignal.connect( sigc::mem_fun( *this, &NewDialog::_cb_create ) );
	
	PushBack( _type );
	PushBack( _plane_div_h );
	PushBack( _plane_div_v );
	PushBack( _create );
}

void NewDialog::_set_plane()
{
	if( _sphere.Value() )
	{
		Erase( Find( &_sphere_div ) );
		Erase( Find( &_create ) );
		PushBack( _plane_div_h );
		PushBack( _plane_div_v );
		PushBack( _create );		
	}
	_sphere.SetValue( false );
	Reorganize();
}
void NewDialog::_set_sphere()
{
	if( _plane.Value() )
	{
		Erase( Find( &_plane_div_h ) );
		Erase( Find( &_plane_div_v ) );
		Erase( Find( &_create ) );
		PushBack( _sphere_div );
		PushBack( _create );
	}
	_plane.SetValue( false );
	Reorganize();
}

void NewDialog::_cb_create()
{
	if( _parent.IsMeshDataSafe() )
	{
		std::string status_text;
		_parent.GetMeshHistory().Clear();
		if( _plane.Value() )
		{
			_parent.GetMeshHistory().GetCurrentMesh().BecomePlane(
				static_cast< unsigned >( _plane_div_h.Value() + 1 ),
				static_cast< unsigned >( _plane_div_v.Value() + 1 ) );
			status_text = "Plane created, ";
		}
		else if( _sphere.Value() )
		{
			_parent.GetMeshHistory().GetCurrentMesh().BecomeSphere(
				static_cast< unsigned >( _sphere_div.Value() ) );
			status_text = "Sphere created, ";
		}
		_parent.View().Show();
		_parent.View().ResetView();
		_parent.RefreshAll();
		status_text += _parent.GetMeshHistory().GetCurrentMesh().GetModelStatisticsString();
		_parent.SetStatusText( status_text );
	}
}
