/* Copyright 2005 Nicholas Bishop
 * 
 * This file is part of SharpConstruct.
 *
 * SharpConstruct is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * SharpConstruct is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SharpConstruct; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#include "SaveDialog.h"
#include "MessageDialog.h"
#include "TopWindow.h"

#include <iostream>

using namespace SharpConstruct::Interface;

SaveDialog::SaveDialog( std::string lbl )
: FileDialog( lbl ), _save( "Save" )
{
	_control_buttons.PushFront( _save );
	
	_save.MouseUpSignal.connect( sigc::mem_fun( *this, &SaveDialog::_done ) );
}

sigc::signal< void, std::string >& SaveDialog::SaveSignal()
{
	return _save_signal;
}

void SaveDialog::_done()
{
	if( FileExists( GetPath() ) )
	{
		std::string r = MessageDialog::ModalMessage( TopWindow::Display(),
			"Save file...", "File '" + FilePart( GetPath() ) +
			"' already exists, overwrite?", "Cancel", "Overwrite" );
		if( r == "Overwrite" )
		{
			SaveSignal()( GetPath() );
			Hide();
		}
	}
	else
	{
		SaveSignal()( GetPath() );
		Hide();
	}
}
