/* Copyright 2005 Nicholas Bishop
 * 
 * This file is part of SharpConstruct.
 *
 * SharpConstruct is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * SharpConstruct is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SharpConstruct; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#include "Scrollbar.h"
#include <GL/gl.h>

using namespace SharpConstruct::Interface;

Scrollbar::Scrollbar( int x, int y, int w, int h )
: Group( x, y, w, h, "" ), _up( 0, 0, 0, 0, "" ),
  _down( 0, 0, 0, 0, "" ), _max( 10 ), _value( 0 )
{
	PushBack( _up );
	PushBack( _down );
	
	MouseDownSignal.connect( sigc::mem_fun( *this, &Scrollbar::_mouse_down ) );
	MouseStillDownSignal.connect( sigc::mem_fun( *this, &Scrollbar::_mouse_down ) );
	MouseMoveSignal.connect( sigc::mem_fun( *this, &Scrollbar::_mouse_down ) );
	ScrollDownSignal.connect( sigc::mem_fun( *this, &Scrollbar::Increment ) );
	ScrollUpSignal.connect( sigc::mem_fun( *this, &Scrollbar::Decrement ) );
	_up.MouseDownSignal.connect( sigc::mem_fun( *this, &Scrollbar::Decrement ) );
	_down.MouseDownSignal.connect( sigc::mem_fun( *this, &Scrollbar::Increment ) );
	_up.MouseStillDownSignal.connect( _up.MouseDownSignal.make_slot() );
	_down.MouseStillDownSignal.connect( _down.MouseDownSignal.make_slot() );
	_up.ScrollDownSignal.connect( sigc::mem_fun( *this, &Scrollbar::Increment ) );
	_up.ScrollUpSignal.connect( sigc::mem_fun( *this, &Scrollbar::Decrement ) );
	_down.ScrollDownSignal.connect( sigc::mem_fun( *this, &Scrollbar::Increment ) );
	_down.ScrollUpSignal.connect( sigc::mem_fun( *this, &Scrollbar::Decrement ) );
}

void Scrollbar::_mouse_down()
{}

VerticalScrollbar::VerticalScrollbar( int x, int y, int w, int h )
: Scrollbar( x, y, w, h )
{
	_up.Set( x, y, w, w );
	_down.Set( x, Bottom() - w, w, w );
}

void VerticalScrollbar::_draw()
{
	Group::_draw();
	
	const float tmp = ( _down.Y() - _up.Bottom() ) * 1.0 / Max();
	//const float min_h = 8.0;
	int h, y;
	
	/*if( tmp < min_h )
	{
		h = min_h;
		const float ny = ( _down.Y() - _up.Bottom() - min_h ) / Max();
		y = _up.Bottom() + ny * Value() + 1; // Fix this...
	}
	else*/
	{
		h = tmp;
		y = _up.Bottom() + tmp * Value() + 1;
	}
	
	// Draw dragger border
	glColor4f( 0.4, 0.4, 0.4, 1 );
	glRecti( X(), y, X() + W(), y + h );

	// Fill dragger
	glColor4f( 0.7, 0.7, 0.7, 1 );
	glRecti( X() + 1, y + 1, X() + W() - 1, y + h - 1 );
	
	// Draw button arrows
	const float frac = W() / 6;
	glColor4f( 0.5, 0.5, 0.5, 1 );
	// Up arrow
	glBegin( GL_TRIANGLES );
	{
		glVertex2f( _up.X() + frac, _up.Y() + frac * 5 );
		glVertex2f( _up.X() + frac * 3, _up.Y() + frac );
		glVertex2f( _up.X() + frac * 5, _up.Y() + frac * 5 );
	} glEnd();
	// Down arrow
	glBegin( GL_TRIANGLES );
	{
		glVertex2f( _down.X() + frac, _down.Y() + frac );
		glVertex2f( _down.X() + frac * 3, _down.Y() + frac * 5 );
		glVertex2f( _down.X() + frac * 5, _down.Y() + frac );
	} glEnd();
}

void VerticalScrollbar::_mouse_down()
{
	if( Event::Mouse.LeftButtonDown )
		SetValue( ( Event::Mouse.Location.Y() - _up.Bottom() ) / ( ( _down.Y() - _up.Bottom() ) * 1.0 / Max() ) );
}
