/* Copyright 2005 Nicholas Bishop
 * 
 * This file is part of SharpConstruct.
 *
 * SharpConstruct is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * SharpConstruct is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SharpConstruct; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#include "MainWindow.h"
#include "Preferences.h"
#include "SettingsDialog.h"

using namespace SharpConstruct::Interface;

SettingsDialog::SettingsDialog( MainWindow& mw )
: Dialog( "Settings" ),
  _parent( mw ),
  _bg_group( true, 0, 0 ),
  _bg_label( "Model background color:" ),
  _bg_reset( "Reset Color" ),
  _bg_color( 100 ),
  _history_steps( "Undo Steps" ),
  _save( "Save" )
{
	_bg_label.AutoResize();
	_bg_group.PushBack( _bg_label );
	_bg_group.PushBack( _bg_reset );
	_bg_group.PushBack( _bg_color );
	
	SetSizeManaged( true );
	
	PushBack( _bg_group );
	PushBack( _history_steps );
	PushBack( _save );
	
	_bg_reset.MouseUpSignal.connect( sigc::mem_fun( *this, &SettingsDialog::_reset_color ) );
	_history_steps.SetProperties( 1, 32, 1 );
	_save.MouseUpSignal.connect( sigc::mem_fun( *this, &SettingsDialog::_save_settings ) );
}

void SettingsDialog::Show()
{
	Dialog::Show();
	
	_bg_color.SetColor( _parent.View().BackgroundColor() );
	_history_steps.Value( _parent.GetMeshHistory().Maximum() - 1 );
}

void SettingsDialog::_reset_color()
{
	_bg_color.SetColor( Color( 0.3, 0.3, 0.3 ) );
}

void SettingsDialog::_save_settings()
{
	_parent.GetMeshHistory().SetMaximum( _history_steps.Value() + 1 );
	_parent.View().SetBackgroundColor( _bg_color.FinalColor() );
	
	Preferences::Instance().Set( "VIEW_COLOR_R", _bg_color.FinalColor().Red );
	Preferences::Instance().Set( "VIEW_COLOR_G", _bg_color.FinalColor().Green );
	Preferences::Instance().Set( "VIEW_COLOR_B", _bg_color.FinalColor().Blue );
	Preferences::Instance().Set( "HISTORY_STEPS", _history_steps.Value() + 1 );
}
