/* Copyright 2004, 2005 Nicholas Bishop
 * 
 * This file is part of SharpConstruct.
 *
 * SharpConstruct is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * SharpConstruct is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SharpConstruct; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#include "Dialog.h"
#include "TopWindow.h"
#include "XDisplayGL.h"

#include <iostream>

using namespace SharpConstruct::Interface;

TopWindow* TopWindow::_display;

TopWindow::TopWindow( int w, int h, std::string lbl )
: Window< Group >( w, h, lbl ), _last_pressed_widget( 0 )
{
	_device = new XDisplayGL( *this, w, h, lbl );
}

TopWindow::~TopWindow()
{
	delete _device;
}

void TopWindow::SetW( int w )
{
	Window< Group >::SetW( w );
	_device->SetSize( W(), H() );
}
void TopWindow::SetH( int h )
{
	Window< Group >::SetH( h );
	_device->SetSize( W(), H() );
}
void TopWindow::Draw()
{
	_device->Draw();
	_display = this;
	Window< Group >::Draw();
	_device->Postwork();
}

void TopWindow::Run()
{	
	const bool BOVINES_NOT_AT_BACKDOOR = true;
	
	while( BOVINES_NOT_AT_BACKDOOR )
	{
		DoEvents();
		Widget::ResetOrderCount();
		Draw();
	}
}
void TopWindow::DoEvents()
{
	_device->DoEvents();
}

void TopWindow::Resize()
{
	Window< Group >::SetW( _device->W() );
	Window< Group >::SetH( _device->H() );
	ResizeSignal()();
}

void TopWindow::MouseDown()
{
	MakeUnfocused();
	Group::MouseDown();
	Widget* rw = ResponderWidgets::Responders.Highest();
	rw->MouseDownSignal.emit();
	rw->MakeActive();
	_last_pressed_widget = ActiveWidget();
	Dialog* rd = ResponderDialogs::Responders.Highest();
	if( rd && rw != rd )
		rd->MouseDownSignal.emit();
}
void TopWindow::MouseStillDown()
{
	Group::MouseStillDown();
	
	Widget* rw = ResponderWidgets::Responders.Highest();
	rw->MouseStillDownSignal.emit();
	rw->MakeActive();
	_last_pressed_widget = ActiveWidget();
}
void TopWindow::MouseUp()
{
	if( ActiveWidget() )
		ActiveWidget()->MakeInactive();
}
void TopWindow::MouseMove()
{
	if( Widget::ActiveWidget() )
	{
		Widget::ActiveWidget()->MouseMoveSignal.emit();
		_device->SetCursor( Widget::ActiveWidget()->Cursor() );
	}
	else
	{
		Group::MouseMove();
		Widget* rw = ResponderWidgets::Responders.Highest();
		if( rw )
		{
			rw->MouseMoveSignal.emit();
			_device->SetCursor( rw->Cursor() );
		}
	}
}
void TopWindow::DoubleClick()
{
	Group::DoubleClick();
	Widget* rw = ResponderWidgets::Responders.Highest();
	if( rw == _last_pressed_widget )
	{
		rw->DoubleClickSignal()();
		rw->MakeActive();
	}
	Dialog* rd = ResponderDialogs::Responders.Highest();
	if( rd && rw != rd )
		rd->DoubleClickSignal()();
}
void TopWindow::ScrollDown()
{
	Group::ScrollDown();
	ResponderWidgets::Responders.Highest()->ScrollDownSignal.emit();
}
void TopWindow::ScrollUp()
{
	Group::ScrollUp();
	ResponderWidgets::Responders.Highest()->ScrollUpSignal.emit();
}
void TopWindow::KeyDown()
{
	if( KeyboardFocus() )
		KeyboardFocus()->KeyDown();
	else
	{
		if( !Group::DoShortcut() )
			Group::KeyDown();
	}
}

sigc::signal< void >& TopWindow::ResizeSignal()
{
	return _resize_signal;
}
