/* Copyright 2005 Nicholas Bishop
 * 
 * This file is part of SharpConstruct.
 *
 * SharpConstruct is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * SharpConstruct is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SharpConstruct; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#include "Dialog.h"
#include "Widget.h"
#include "WidgetList.h"
#include <GL/gl.h>

using namespace SharpConstruct::Interface;

ResponderWidgets ResponderWidgets::Responders;
ResponderDialogs ResponderDialogs::Responders;
Widget* Widget::_active_widget = 0;
Widget* Widget::_keyboard_focus = 0;
int Drawable::_order_count = 0;

Widget* ResponderWidgets::Highest()
{
	if( Size() == 0 )
		return 0;
	Widget* ndx = Front();
	for( Iterator i = Begin(); i != End(); i++ )
	{
		if( ( *i )->Order() > ndx->Order() )
			ndx = ( *i );
	}
	return ndx;
}

Dialog* ResponderDialogs::Highest()
{
	if( Size() == 0 )
		return 0;
	Dialog* ndx = Front();
	for( Iterator i = Begin(); i != End(); i++ )
	{
		if( ( *i )->Order() > ndx->Order() )
			ndx = ( *i );
	}
	return ndx;
}

Widget* Widget::ActiveWidget()
{
	return _active_widget;
}
bool Widget::Active() const
{
	return ActiveWidget() == this;
}
void Widget::MakeActive()
{
	_active_widget = this;
}
void Widget::MakeInactive()
{
	if( Contains() )
		MouseUpSignal.emit();
	_active_widget = 0;
}
Widget* Widget::KeyboardFocus()
{
	return _keyboard_focus;
}
bool Widget::Focused() const
{
	return _keyboard_focus == this;
}
void Widget::MakeFocused()
{
	_keyboard_focus = this;
}
void Widget::MakeUnfocused()
{
	_keyboard_focus = 0;
}

bool Widget::Enabled() const
{
	return _enabled;
}
void Widget::SetEnabled( bool e )
{
	_enabled = e;
}

std::string Widget::Label() const
{
	return _label;
}
void Widget::SetLabel( std::string lbl )
{
	_label = lbl;
}

int Widget::Border() const
{
	return _border;
}
void Widget::SetBorder( int b )
{
	_border = b;
}

KeyData Widget::Shortcut() const
{
	return _shortcut;
}
void Widget::SetShortcut( KeyData key )
{
	_shortcut = key;
}
Widget::Cursors Widget::Cursor() const
{
	return _cursor;
}
void Widget::SetCursor( Cursors c )
{
	_cursor = c;
}

bool Widget::Contains( const Math::Point2D< int >& in ) const
{
	return Math::Rect2D< int >::Contains( in );
}
bool Widget::Contains() const
{
	return Contains( Event::Mouse.Location );
}

void Widget::Show()
{
	SetVisible( true );
}
void Widget::Hide()
{
	SetVisible( false );
}
void Widget::ToggleVisible()
{
	if( Visible() )
		Hide();
	else
		Show();
}

sigc::signal< void >& Widget::DoubleClickSignal()
{
	return _double_click_signal;
}

void Widget::MouseDown()
{
	if( Enabled() && Visible() && Contains( Event::Mouse.Location ) )
		ResponderWidgets::Responders.PushBack( this );
}
void Widget::MouseStillDown()
{
	if( Enabled() && Visible() && Contains( Event::Mouse.Location ) )
		ResponderWidgets::Responders.PushBack( this );
}
void Widget::MouseUp()
{				
	if( Active() && Enabled() && Visible() && Contains( Event::Mouse.Location ) )
		ResponderWidgets::Responders.PushBack( this );
	
	/*if( Enabled() && Contains( Event::Mouse.Location ) )
		WidgetList< Widget* >::MouseStillDown();*/ // What the hell is this?
}
void Widget::DoubleClick()
{
	if( Enabled() && Visible() && Contains( Event::Mouse.Location ) )
		ResponderWidgets::Responders.PushBack( this );
}
void Widget::MouseMove()
{
	if( Enabled() && Visible() && Contains( Event::Mouse.Location ) )
		ResponderWidgets::Responders.PushBack( this );
}
void Widget::ScrollDown()
{
	if( Enabled() && Visible() && Contains( Event::Mouse.Location ) )
		ResponderWidgets::Responders.PushBack( this );
}
void Widget::ScrollUp()
{
	if( Enabled() && Visible() && Contains( Event::Mouse.Location ) )
		ResponderWidgets::Responders.PushBack( this );
}
void Widget::KeyDown()
{
}
bool Widget::DoShortcut()
{
	if( Enabled() && _shortcut == Event::Keyboard )
	{
		MouseUpSignal();
		return true;
	}
	else
		return false;
}

void Widget::AutoResize()
{
	SetSize( ( Label().length() + 1 ) * TEXT_WIDTH, CONTROL_HEIGHT );
}

void Drawable::Draw()
{
	if( Visible() )
	{
		_order = _order_count;
		_order_count++;
		_draw();
	}
}

void Drawable::_draw()
{}
