# $Id: Config.pm,v 1.8 2005/04/03 12:49:40 verthezp Exp $
# $Name: R2_04 $
#___________________________________________________________________
#
# Configuration routines for intclock
#___________________________________________________________________

# Copyright (C) 2004, 2005 by Peter Verthez

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.

# Written by Peter Verthez, <Peter.Verthez@advalvas.be>.
# Based on Jens-Ulrik Petersen's hsclock

package Intclock::Config;

use strict;
use diagnostics;
use Data::Dumper;
use Safe;
use vars qw($config_utf8_on $config_default_gui $config_dd_fix_utf8
	    @config_guis $config_datadir $config_strftime_fix_utf8);

use vars qw($gui $progname $configfile @all_guis $datadir $strftime_fix_utf8);

BEGIN {
  require "Intclock.pl";
  if ($config_utf8_on) {
    require Encode;
  }
  $gui = $config_default_gui;
  @all_guis = @config_guis;
  $datadir = $config_datadir;
  $strftime_fix_utf8 = $config_strftime_fix_utf8;
}

sub get_datadir {
  if (-r "data/intclock.png") {
    return "data";
  }
  else {
    return $datadir;
  }
}

# Fixing bad utf8 behaviour of Data::Dumper and Gtk
sub fix_utf8 {
  my ($string) = @_;
  if ($config_utf8_on and $ENV{"LANG"} =~ /utf\-?8/i) {
    Encode::_utf8_on($string);
  }
  return $string;
}

sub init {
  my ($program) = @_;
  $progname = $program;
  $configfile = "$ENV{HOME}/.$progname";
}

sub load {
  my $cpt = new Safe;
  my $config;
  $cpt->permit(qw(:base_core));
  if (-r $configfile) {
    $config = $cpt->rdo("$configfile");
  }
  return $config;
}

sub save {
  my ($config) = @_;
  my $d = new Data::Dumper([$config]);
  $d->Terse(1);
  open OUTPUT, ">$configfile" or return undef;
  print OUTPUT "# Auto-generated by $progname, don't edit by hand !\n";
  my $output = $d->Dump();
  if ($config_dd_fix_utf8) {
    $output = fix_utf8($output);
  }
  print OUTPUT $output;
  close OUTPUT;
}

1;
