# $Id: UI.pm,v 1.5 2005/04/02 11:55:31 verthezp Exp $
# $Name: R2_04 $
#___________________________________________________________________
#
# User interface routines for intclock
#___________________________________________________________________

# Copyright (C) 2004, 2005 by Peter Verthez

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.

# Written by Peter Verthez, <Peter.Verthez@advalvas.be>.
# Based on Jens-Ulrik Petersen's hsclock

package Intclock::UI;

use strict;
use diagnostics;
use Time::HiRes;
use Intclock::TZ;
use vars qw($debug @timeformats $timeformat %zonenames $use_names
	    $use_usernames $hide_vowels %tf_configs);

BEGIN {
  $debug = 0;
  @timeformats = ("%a %d%n%H:%M:%S",
		  "%a %d%n%I:%M:%S %p",
		  "%H:%M:%S",
		  "%I:%M:%S %p",
		  "%a %d%n%H:%M",
		  "%a %d%n%I:%M %p",
		  "%H:%M",
		  "%I:%M %p",
		  );
  $timeformat = $timeformats[0];
  %tf_configs = ( "%a %d%n%H:%M:%S"    => [ 1, 1, 1 ],
		  "%a %d%n%I:%M:%S %p" => [ 1, 0 ,1 ],
		  "%H:%M:%S"           => [ 1, 1, 0 ],
		  "%I:%M:%S %p"        => [ 1, 0, 0 ],
		  "%a %d%n%H:%M",      => [ 0, 1, 1 ],
		  "%a %d%n%I:%M %p"    => [ 0, 0 ,1 ],
		  "%H:%M"              => [ 0, 1, 0 ],
		  "%I:%M %p"           => [ 0, 0, 0 ],
		  );
		  
  $use_names = 0;
  $hide_vowels = 0;
}

sub set_config {
  my ($config, $cmdline) = @_;

  $debug      = $cmdline->{"debug"} if exists $cmdline->{"debug"};
  $timeformat = $config->{"format"} if exists $config->{"format"};
  debug("CONFIG: timeformat = $timeformat");
  if (exists $config->{"name"}) {
    $use_names     = ( $config->{"name"} == 1 ? 1 : 0 );
    $use_usernames = ( $config->{"name"} == 2 ? 1 : 0 );
  }
  $hide_vowels = $config->{"vowel"} if exists $config->{"vowel"};
  
  foreach my $zone (@{$config->{"zones"}}) {
    my ($zonename) = @$zone;
    my @names = Intclock::TZ::get_timezone_names($zonename);
    my @zoneparts = split "/", $zonename;
    $zonenames{$zonename} = [$zoneparts[-1], $names[0], $names[1]];
  }
}

sub debug {
  my ($txt) = @_;
  print "$txt\n" if $debug;
}

sub seconds_needed {
  return ($timeformat =~ /\%.?[STXcrs]/);
}

sub date_time_string {
  my ($localtime_array) = @_;
  return POSIX::strftime($timeformat, @$localtime_array);
}

sub timezone_label {
  my ($tz, $t_vals) = @_;
  my ($zonename, $username) = @$tz;
  if ($use_names or $use_usernames) {
    my $str = $zonenames{$zonename}[0];
    $str =~ s/_/ /g;
    $str =~ s/[aeiou]//g if $hide_vowels;
    if ($use_usernames) {
      return $username || $str;
    }
    else {
      return $str;
    }
  }
  else {
    return $zonenames{$zonename}[$t_vals->[8] ? 2 : 1];
  }
}

sub get_timer_interval {
  my ($t_s, $t_m) = Time::HiRes::gettimeofday();
  debug($t_m);
  my $seconds = 1;
  if (! seconds_needed()) {
    my ($seconds_in_minute) = gmtime($t_s);
    $seconds = 60 - $seconds_in_minute;
  }
  my $timer = $seconds * 1000 - $t_m / 1000 + 10;
  debug("TO: $timer");
  return $timer;
}

1;
