//
// C++ Implementation: passivepopup
//
// Description:
//
//
// Author:  Pieter Pareit <pieter.pareit@scarlet.be>, (C) 2005
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "passivepopup.h"

#include <kdialog.h>
#include <kcursor.h>
#include <kapplication.h>

#include <qfont.h>
#include <qvbox.h>
#include <qhbox.h>
#include <qlabel.h>

PassivePopup::PassivePopup(QWidget *parent, const char *name)
    : KPassivePopup(parent, name),
    m_captionFont( new QFont() ),
    m_textFont( new QFont() ),
    m_dragging( false ),
    m_dragOffset( QPoint() ),
    m_oldTimeoutDelay( -2 )
{
  m_captionFont->setBold( true );
}


PassivePopup::~PassivePopup()
{
  delete m_captionFont;
  delete m_textFont;
}

void PassivePopup::setCaptionFont( const QFont& font)
{
  *m_captionFont = font;
}

void PassivePopup::setTextFont( const QFont& font)
{
  *m_textFont = font;
}

void PassivePopup::setView( const QString& caption, const QString& text, const QPixmap& icon )
{
  //  |icon|caption|
  //  |  text      |

  QVBox *vbox = new QVBox( this );
  vbox->setSpacing( KDialog::spacingHint() );

  QHBox *hbox = new QHBox( vbox );
  hbox->setMargin(0);
  hbox->setSpacing(KDialog::spacingHint() );

  QLabel *iconLabel = new QLabel( hbox, "icon" );
  iconLabel->setPixmap( icon );
  iconLabel->setAlignment( AlignLeft );

  if ( !caption.isEmpty() )
  {
    QLabel *captionLabel = new QLabel( caption, hbox, "caption" );
    captionLabel->setFont( *m_captionFont );
    captionLabel->setAlignment( Qt::AlignHCenter );
    hbox->setStretchFactor( captionLabel, 10 ); // enforce centering
  }

  if ( !text.isEmpty() )
  {
    QLabel *textLabel = new QLabel( text, vbox, "text" );
    textLabel->setFont( *m_textFont );
    textLabel->setAlignment( Qt::AlignLeft );
  }

  // give the view to our base class
  KPassivePopup::setView( vbox );
}

void PassivePopup::mousePressEvent( QMouseEvent* e )
{
  if( e->button() == LeftButton && !m_dragging )
  {
    m_oldTimeoutDelay = timeout();
    // @fixme: setTimeout( 0 ) should be possible, but then popup disapears on mousePress, not on mouseRelease
    setTimeout( 0xFFFF );
    m_dragOffset = e->pos();
    grabMouse( KCursor::sizeAllCursor() );
    m_dragging = true;
    emit draggingStarted();
  } else if ( e->button() == RightButton ) {
    this->hide();
  }
}

void PassivePopup::mouseReleaseEvent( QMouseEvent* e )
{
  if( m_dragging )
  {
    setTimeout( m_oldTimeoutDelay );
    m_dragging = false;
    releaseMouse();
    emit draggingEnded();
    emit draggingEnded( pos() );
  }
}

void PassivePopup::mouseMoveEvent( QMouseEvent* e )
{
  if( m_dragging && this == mouseGrabber() )
  {
    const QRect screen = KApplication::desktop()->screenGeometry( );
    QPoint destination = e->globalPos() - m_dragOffset - screen.topLeft();
    move( destination );
  } 
}

void PassivePopup::positionSelf()
{
  // we override this, so it does not reset its position each time
}

#include "passivepopup.moc"
